<?php
/**
 * Webtexttool - SEO made easy, for everyone
 *
 * @author       Jan Martin Roelofs
 * @copyright    (c) Webtexttool - 2019
 * @package      webtexttool
 * @license      http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version      2.0.1
 * @date         2019-04-06
 *
 */

defined('_JEXEC') or die;

class com_webtexttoolInstallerScript
{
    /*
     * $type is the type of change (install, update or discover_install, not uninstall).
     */
    function preflight( $type, $parent ) {
        $jversion = new JVersion();

        // Installing component manifest file version
        $this->release = $parent->get( "manifest" )->version;

        // Manifest file minimum Joomla version
        $this->minimum_joomla_release = $parent->get( "manifest" )->attributes()->version;

        // Show the essential information at the install/update back-end
        echo '<p>Installing component manifest file version = ' . $this->release;
        echo '<br />Installing component manifest file minimum Joomla version = ' . $this->minimum_joomla_release;
        echo '<br />Current Joomla version = ' . $jversion->getShortVersion();

        // abort if the current Joomla release is older
        if( version_compare( $jversion->getShortVersion(), $this->minimum_joomla_release, 'lt' ) ) {
            Jerror::raiseWarning(null, 'Cannot install Webtexttool in a Joomla release prior to '.$this->minimum_joomla_release);
            return false;
        }

        // abort if the component being installed is not newer than the currently installed version
        if ( $type == 'update' ) {
            $oldRelease = $this->getParam('version');
            $rel = $oldRelease . ' to ' . $this->release;
            if ( version_compare( $this->release, $oldRelease, 'le' ) ) {
                Jerror::raiseWarning(null, 'Incorrect version sequence. Cannot upgrade ' . $rel);
                return false;
            }
        }
        else { $rel = $this->release; }
    }

    function install( $parent ) {
        $parent->getParent()->setRedirectURL('index.php?option=com_webtexttool');
    }

    function update( $parent ) {
        // $parent->getParent()->setRedirectURL('index.php?option=com_webtexttool');
    }

    function postflight( $type, $parent ) {
        $this->wtt_plugin_install();
        $this->wtt_plugin_enable();
        $this->wtt_social_plugin_install();
        $this->wtt_social_plugin_enable();
        $this->wtt_add_fields();
    }

    function uninstall( $parent ) {
        $this->wtt_plugin_disable();
        $this->wtt_social_plugin_disable();
        echo '<p>' . JText::_('COM_WEBTEXTTOOL_UNINSTALLED') . '</p>';
    }

    /*
     * install the plugin
     */
    function wtt_plugin_install() {
        $installer =  new JInstaller();
        $path = dirname(__FILE__) . '/plugin';
        $installer->install($path);
    }

    function wtt_social_plugin_install() {
        $installer =  new JInstaller();
        $path = dirname(__FILE__) . '/social';
        $installer->install($path);
    }

    function wtt_plugin_enable() {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 1 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttool'));
        $db->setQuery($query);
        $result = $db->query();
    }

    function wtt_plugin_disable() {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 0 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttool'));
        $db->setQuery($query);
        $result = $db->query();
    }
    function wtt_social_plugin_enable() {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 1 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttoolsocial'));
        $db->setQuery($query);
        $result = $db->query();
    }

    function wtt_social_plugin_disable() {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 0 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttoolsocial'));
        $db->setQuery($query);
        $result = $db->query();
    }

    function wtt_add_fields() {
        $ourFields = array (
            array('title' => 'wtt_page_score'),
            array('title' => 'wtt_post_languagecode'),
            array('title' => 'wtt_post_synonyms'),
            array('title' => 'wtt_content_quality_suggestions'),
            array('title' => 'wtt_content_quality_settings')
        );
        $ourBooleanFields = array (
            // array('title' => 'wtt_process_page_title_as_h1')
        );
        JLoader::register('FieldsModelField', JPATH_ADMINISTRATOR . '/components/com_fields/models/field.php');
        $ourFieldsModelField = new FieldsModelField;
        $aField = array (
            'id' => 0,
            'context' => '',
            'group_id' => '0',
            'assigned_cat_ids' => 
            array (
            0 => '',
            ),
            'title' => '',
            'name' => '',
            'type' => 'text',
            'required' => '0',
            'default_value' => '',
            'state' => '1',
            'created_user_id' => '0',
            'created_time' => NULL,
            'modified_time' => NULL,
            'language' => '*',
            'note' => '',
            'label' => '',
            'description' => '',
            'access' => '1',
            'rules' => 
            array (
            'core.delete' => 
            array (
            ),
            'core.edit' => 
            array (
            ),
            'core.edit.state' => 
            array (
            ),
            'core.edit.value' => 
            array (
            ),
            ),
            'params' => 
            array (
            'hint' => '',
            'class' => '',
            'label_class' => '',
            'show_on' => '2',
            'render_class' => '',
            'showlabel' => '1',
            'label_render_class' => '',
            'display' => '0',
            'layout' => '',
            'display_readonly' => '2',
            ),
            'fieldparams' => 
            array (
            'filter' => '',
            'maxlength' => '',
            ),
            'tags' => NULL,
        );
        foreach ($ourFields as $Field) {
            $aField['title'] = $Field['title'];
            $aField['label'] = $Field['title'];
            $aField['params']['class'] = $Field['title'];
            $aField['context'] = 'com_content.article';
            $ourFieldsModelField->save($aField);
            $aField['title'] = $Field['title'] . '_category';
            $aField['label'] = $Field['title'];
            $aField['context'] = 'com_content.categories';
            $ourFieldsModelField->save($aField);
        } 
        foreach ($ourBooleanFields as $Field) {
            $aField['type'] = 'checkboxes';
            $aField['fieldparams'] = array (
                'options' => 
                array (
                    'options0' => 
                    array (
                        'name' => '',
                        'value' => '',
                    ),
                ),
            );
            $aField['title'] = $Field['title'];
            $aField['label'] = $Field['title'];
            $aField['params']['class'] = $Field['title'];
            $aField['context'] = 'com_content.article';
            $ourFieldsModelField->save($aField);
            $aField['title'] = $Field['title'] . '_category';
            $aField['label'] = $Field['title'];
            $aField['context'] = 'com_content.categories';
            $ourFieldsModelField->save($aField);
        } 
    }

    /*
     * get a variable from the manifest file (actually, from the manifest cache)
     */
    function getParam( $name ) {
        $db = JFactory::getDbo();
        $db->setQuery('SELECT manifest_cache FROM #__extensions WHERE name = "com_webtexttool"');
        $manifest = json_decode( $db->loadResult(), true );
        return $manifest[ $name ];
    }

}