<?php
/**
 * Webtexttool - SEO made easy, for everyone
 *
 * @author       Jan Martin Roelofs
 * @copyright    (c) Webtexttool - 2019
 * @package      webtexttool
 * @license      http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version      2.0.1
 * @date         2019-04-06
 *
 */

defined('_JEXEC') or die;

class Webtexttool
{

    protected $plugin_name;

    protected $version;

    const wtt_short_url = "api.webtexttool.com";
    const wtt_base_url = "https://api.webtexttool.com/";
    const wtt_api_base_url = "https://api.webtexttool.com";
    const wtt_app_url = "https://app.webtexttool.com/";
    const wtt_action = "webtexttool";
    const wtt_version = "2.0.1";

    public function __construct()
    {
        $this->plugin_name = 'webtexttool';
        $this->version = wtt_version;
    }

    public static function get_plugin_path()
    {
        return JURI::root(true) . '/plugins/system/webtexttool/';
    }

    public static function get_component_path()
    {
        return JURI::root(true) . '/administrator/components/com_webtexttool/';
    }

    public static function get_admin_url()
    {
        return JURI::root(true) . '/administrator/index.php?option=com_webtexttool';
    }

    public static function get_plugin_name()
    {
        return $this->plugin_name;
    }

    public static function get_page_url()
    {
        $u = JURI::getInstance( JURI::base() );
        return $u->getScheme() . '://' . $u->getHost() .
            str_replace('&layout=edit', '', str_replace('/administrator', '', JRequest::getURI()));
    }

    public static function get_site_url()
    {
        return JURI::root();
    }

    public static function get_version()
    {
        return $this->version;
    }

    public static function get_language()
    {
        $user = JFactory::getUser();
        $lang = JFactory::getLanguage();
        $lang_code = $user->getParam('language', $lang->getTag());
        return substr($lang_code, 0, 2);
    }

    public static function get_allowed_title_tags()
    {
        // the empty one has to come first
        return array("No article title used", "h1", "h2", "h3", "h4");
    }

    public static function set_title_tag($titleTag){
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__webtexttool_heading'))
            ->set   ($db->quoteName('heading_type') . ' = ' . $db->quote($titleTag))
            ->where ($db->quoteName('id') . ' = ' . (int) 1);
        $db->setQuery($query);
        return $db->query();
    }

    public static function get_title_tag(){
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select($db->quoteName('heading_type'))
            ->from  ($db->quoteName('#__webtexttool_heading'))
            ->where ($db->quoteName('id') . ' = ' . (int) 1);
        $db->setQuery($query);
        return $db->loadResult();
    }

    public static function wtt_social_plugin_enable() {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 1 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttoolsocial'));
        $db->setQuery($query);
        $result = $db->query();
    }

    public static function wtt_social_plugin_disable() {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->update($db->quoteName('#__extensions'))
            ->set   ($db->quoteName('enabled') . ' = ' . (int) 0 )
            ->where ($db->quoteName('element') . ' = ' . $db->quote('webtexttoolsocial'));
        $db->setQuery($query);
        $result = $db->query();
    }

    public static function wtt_social_plugin_get_state() {
        return JPluginHelper::isEnabled('system', 'webtexttoolsocial');
    }

}
