var app = angular.module("wttDashboard", ['tc.chartjs', 'ngSanitize', 'wtt.ui.bootstrap', 'ngCookies', 'cgBusy', 'scrollable-table', 'toastr', 'ngTagsInputWtt', 'angular-ladda']);

app.config(['$httpProvider', 'laddaProvider', 'toastrConfig',
    function($httpProvider, laddaProvider, toastrConfig) {
        $httpProvider.interceptors.push('authInterceptor');

        angular.extend(toastrConfig, {
            maxOpened: 3
        });

        laddaProvider.setOption({
            style: "expand-left"
        });
    }
]);

app.controller("editPageController", ['$scope', '$http', '$q', 'stateService', '$timeout', '$interval', '$cookies', 'suggestionsService', 'keywordService', 'httpService', 'languageService', '$sce', 'toastr', 'synonymService',
    function($scope, $http, $q, stateService, $timeout, $interval, $cookies, suggestionsService, keywordService, httpService, languageService, $sce, toastr, synonymService) {
        var WttApiBaseUrl = wtt_globals.wtt_base_api_url,
            $j = jQuery;
        if (!wtt_globals.metaKeyFieldShown) {
            console.error("Publishing options are not reachable!");
            toastr.error("Please set 'Show Publishing Options' to Yes!", {
                closeButton: !0,
                allowHtml: !0,
                timeOut: 0,
                extendedTimeOut: 0
            });
        }

        httpService.getData(WttApiBaseUrl + "user/authenticated").then(function(result) {
            $scope.auth = result;

            if (result) {
                httpService.getData(WttApiBaseUrl + "user/info").then(function(userInfo) {
                    $scope.userInfo = userInfo;
                    $scope.runRules = false;
                    $scope.runRulesTimeout = 2000;
                    $scope.isCollapsed = false;
                    $scope.keywordSet = false;
                    $scope.ruleSet = 1;
                    $scope.activeEngine = 'seo';
                    $scope.HtmlContent = "";
                    $scope.isReadingLevelDetailsCollapsed = true;
                    var selectedPageNodes = [];
                    var keywordSynonyms = null;

                    var synonymList = (wtt_globals.wtt_post_synonyms.value ? JSON.parse(wtt_globals.wtt_post_synonyms.value) : '');

                    function removeSpecialChars(string) {
                        return string.replace(/[`~!@#$%^&*()_|+=?;:,.<>\{\}\[\]\\\/]/gi, '');
                    }

                    $scope.pageKeywordSynonyms = synonymList;

                    $scope.getKeywordSynonyms = function(query) {

                        var synonymsDeferred = $q.defer();

                        if (_.isNull(keywordSynonyms)) {

                            if ($j('#wtt-keyword').val() !== "") {
                                var localKeyword = $j.trim(cleanKeyword(removeSpecialChars(getContent("wtt-keyword"))));

                                synonymService.getSynonyms(localKeyword)
                                    .then(function(rawSynonymsData) {
                                        keywordSynonyms = _.pluck(rawSynonymsData.Associations, 'Synonym');
                                        synonymsDeferred.resolve(keywordSynonyms);
                                    });
                            }
                        } else {
                            synonymsDeferred.resolve(keywordSynonyms);
                        }

                        return synonymsDeferred.promise;
                    };

                    $scope.onSynonymsAdded = function(synonym) {
                        if ($j('#wttSynonymTags li').length <= 2) {
                            $j("#wttSynonymTags").append('<li><input type="hidden" id="wtt-synonym-tags" name="wtt_synonym_tags[]" value=\"' + synonym.text + '\"/></li>');
                        }
                        $scope.saveSynonyms();
                    };

                    $scope.onSynonymsRemoved = function(synonym) {
                        $j('input[value=\"' + synonym.text + '\"]').closest('li').remove();
                        $scope.saveSynonyms();
                    };

                    $scope.saveSynonyms = function() {
                        var synonymArray = $j.map($scope.pageKeywordSynonyms, function(element) {
                            return element.text;
                        });
                        if (synonymArray.length) {
                            wtt_globals.wtt_post_synonyms.value = JSON.stringify(synonymArray);
                        } else {
                            wtt_globals.wtt_post_synonyms.value = '';
                        }
                    };

                    function getContent(id) {
                        return $j('#' + id).val();
                    }

                    if (userInfo.UserState === 3 && userInfo.TrialDays < 0) {
                        toastr.warning('Upgrade your account <a href="https://app.webtexttool.com/#/register-paid" target="_blank">here</a>', "Your trial account has expired!", {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });
                    }

                    $scope.htmlPopover = $sce.trustAsHtml('<p class="tooltip-content">Select for which language/country you want to optimize your content.<br><br>Enter your keyword.<br><br>Add synonyms (optional).</p>');
                    $scope.htmlPopoverS = $sce.trustAsHtml('<p class="tooltip-content">While writing, multiple suggestions appear here. These suggestions tell you how you can improve your text for the search engines, according to the latest SEO rules, but also how to structure your text for your readers. Following these suggestions, will raise your optimization score!</p>');
                    $scope.htmlPopoverP = $sce.trustAsHtml('<p class="tooltip-content">Here you can set max 3 alternative keywords that support the main keyword.</p>');

                    var wpPermalink = wtt_globals.permalink;

                    if (wpPermalink !== "") {
                        $scope.permalink = wpPermalink.replace(/-/g, ' ');
                    }

                    var domainUrl = wtt_globals.siteUrl;
                    if (domainUrl !== null) {
                        $scope.domainUrl = getDomain(domainUrl);
                    }

                    function getDomain(url) {
                        var hostName = getHostName(url);
                        var domain = hostName;

                        if (hostName !== null) {
                            var parts = hostName.split('.').reverse();

                            if (parts !== null && parts.length > 1) {
                                domain = parts[1] + '.' + parts[0];

                                if (hostName.toLowerCase().indexOf('.co.uk') != -1 && parts.length > 2) {
                                    domain = parts[2] + '.' + domain;
                                }
                            }
                        }
                        return domain;
                    }

                    function getHostName(url) {
                        var match = url.match(/:\/\/(www[0-9]?\.)?(.[^/:]+)/i);
                        if (match !== null && match.length > 2 && typeof match[2] === 'string' && match[2].length > 0) {
                            return match[2];
                        } else {
                            return null;
                        }
                    }

                    var languageOfDoc = document.getElementById('wtt-language-code-field').value;

                    if ((userInfo.DefaultLanguageCode != languageOfDoc) && (['nl', 'de', 'en'].indexOf(languageOfDoc) > -1)) {
                        $scope.activeLanguageCode = languageOfDoc;
                        updateUserLanguage(languageOfDoc);
                        $cookies.put('wtt_lang', languageOfDoc);
                        $scope.localLanguageCode = languageOfDoc;
                    } else {
                        $cookies.put('wtt_lang', userInfo.DefaultLanguageCode);
                        $scope.localLanguageCode = userInfo.DefaultLanguageCode;
                    }

                    $scope.checkboxModel = {
                        isChecked: false
                    };

                    var updateWttContentDefault = function() {
                        if (typeof tinymce !== "undefined" && tinymce.activeEditor) {

                            var updateTimeout = 1000;
                            var editor = tinymce.activeEditor;
                            // console.log('before not dirty');
                            editor.isNotDirty = false;
                            // check for jce editor
                            if (tinymce.settings && tinymce.settings.theme == 'advanced') {

                                $j(editor.getContentAreaContainer()).on('mouseenter', function() {
                                    broadcastZeroNodes().then(function() {
                                        selectedPageNodes = ['MAINCONTENT'];
                                        broadcastSelectedNodes();
                                    });
                                });

                                $j(editor.getContentAreaContainer()).on('mouseleave', function() {
                                    broadcastZeroNodes();
                                });

                            } else {

                                editor.on('focus', function() {
                                    broadcastZeroNodes().then(function() {
                                        selectedPageNodes = ['MAINCONTENT'];
                                        broadcastSelectedNodes();
                                    });
                                });

                                editor.on('blur', function() {
                                    broadcastZeroNodes();
                                });

                            }

                            var updateIfDirty = $interval(function() {
                                if (editor.isDirty() && !editor.isNotDirty) {

                                    $scope.HtmlContent = editor.getContent();
                                    $scope.updateSuggestions();
                                    editor.isNotDirty = true;

                                }
                            }, updateTimeout);

                            if (editor.isNotDirty) {
                                $interval.cancel(updateIfDirty);
                            }
                        } else {
                            console.error("TinyMCE editor is not available!");
                            toastr.warning('Switch the editor on to continue optimizing your content.', 'You might have disabled the TinyMCE or JCE editor.', {
                                closeButton: true,
                                timeOut: 0,
                                extendedTimeOut: 0
                            });
                        }
                    };

                    $scope.useMyKeyword = function() {
                        $scope.errorMsg = null;

                        function getHtmlAndRunSuggestions() {
                            $scope.runRules = true;
                            $scope.runSuggestions();
                            updateWttContentDefault();
                        }

                        if ($scope.pageHasKeyword()) {
                            if (!validateKeyword()) {
                                toastr.warning($scope.errorMsg, {
                                    closeButton: true,
                                    timeOut: 3000
                                });
                                return;
                            }

                            $scope.Keyword = getContent('wtt-keyword');
                            $scope.Title = getContent(wtt_globals.titleFieldName);
                            $scope.Description = getContent(wtt_globals.metaDescFieldName);

                            if (typeof(tinymce) == "undefined") {
                                toastr.error("Webtexttool plugin needs a TinyMCE compatible editor!", {
                                    closeButton: !0,
                                    allowHtml: !0,
                                    timeOut: 0,
                                    extendedTimeOut: 0
                                });
                            } else {
                                if (tinymce.activeEditor) {
                                    $scope.HtmlContent = tinymce.activeEditor.getContent() || "";
                                    getHtmlAndRunSuggestions();
                                } else {
                                    $timeout(function() {
                                        if (tinymce.activeEditor) {
                                            $scope.HtmlContent = tinymce.activeEditor.getContent() || "";
                                            getHtmlAndRunSuggestions();
                                        } else {
                                            $timeout(function() {
                                                if (tinymce.activeEditor) {
                                                    $scope.HtmlContent = tinymce.activeEditor.getContent() || "";
                                                } else {
                                                    $scope.HtmlContent = '';
                                                    console.error("tinymce has not been initialized");
                                                }
                                                getHtmlAndRunSuggestions();
                                            }, 2000);
                                        }
                                    }, 1000);
                                }
                            }

                            $timeout(function() {
                                $scope.isCollapsed = true;
                                $scope.keywordSet = true;
                            }, 500);
                        } else {
                            toastr.warning("Keyword is required!");
                        }
                    };

                    function replaceLineBreaks(str) {
                        return str.replace(/(?:\r\n|\r|\n)/g, '');
                    }

                    /**
                     * Content Quality module implementation - contentQualityController.js
                     */
                    $scope.updateSuggestionsStart = function() {
                        $interval($scope.runSuggestions, $scope.runRulesTimeout);
                    };

                    $scope.rulesRunning = false;
                    var suggestionsRanFirstTime = false;

                    $scope.seoClass = "page-score";
                    $scope.contentClass = "gray-score";

                    /*if(typeof Chart.defaults.global.tooltips.enabled !== "undefined") {
                        Chart.defaults.global.tooltips.enabled = false;
                    }*/

                    $scope.pageScoreChartOptions = {
                        segmentShowStroke: false,
                        showTooltips: false,
                        cutoutPercentage: 70,
                        responsive: true,
                        maintainAspectRatio: false,
                        animateRotate: false
                    };

                    var buildActiveChart = function(score) {
                        return {
                            datasets: [{
                                borderColor: ["#5cb85c", "#cee9ce"],
                                data: [score, 100 - score],
                                backgroundColor: ["#5cb85c", "#eee"]
                            }]
                        };
                    };

                    var buildInactiveChart = function(score) {
                        return {
                            datasets: [{
                                borderColor: ["#D5D5D5", "#D5D5D5"],
                                data: [score, 100 - score],
                                backgroundColor: ["#D5D5D5", "#eee"]
                            }]
                        };
                    };


                    function isNullOrEmpty(str) {
                        return (str ? str : "").trim().length === 0;
                    }

                    $scope.analyze = function() {
                        var minValue = 1;

                        if (isNullOrEmpty($scope.HtmlContent)) {
                            toastr.error("Content is required");
                            return;
                        }

                        if ($scope.userInfo.Features.indexOf("ContentQuality") >= 0 && $scope.userInfo.Credits >= minValue) {

                            // update page settings QualityLevels
                            $scope.QualityLevels = $scope.settings;

                            //Save content quality settings to wp database
                            saveContentQualitySettings($scope.QualityLevels);

                            analyzeContentQuality(20);
                        } else {
                            toastr.warning("Text Conversion Optimizer is not included in your subscription. Please upgrade or contact our support team for more info.", "Oops.. You can't do this now. This might not be included in your current webtexttool plan or you have run out of credits for this month.", {
                                closeButton: true,
                                timeOut: 0,
                                extendedTimeOut: 3000
                            });
                        }
                    };

                    function getDefaultQualitySettings() {
                        return {
                            ReadingLevel: "1",
                            DifficultWordsLevel: 1,
                            LongSentencesLevel: 1,
                            AdjectivesLevel: 1,
                            WhitespacesLevel: 1,
                            BulletPointsLevel: 1,
                            ImagesLevel: 1,
                            GenderLevel: 'n',
                            SentimentLevel: 'neutral',

                            GenderList: 1,
                            JargonList: 1,
                            SingularPluralLevel: 1,
                            RepetitionLevel: 1,
                            TextLengthRuleLevel: 1
                        };
                    }

                    $scope.data = stateService.data;

                    $scope.data.Resources.forEach(function(el, i) {
                        $scope.data.Resources[el.ResourceKey] = $sce.trustAsHtml(el.HtmlContent);
                    });

                    $scope.showReadingLevelHelp = $scope.data.Resources.LanguageLevelHelp.toString();

                    function GetPageContent() {
                        var content = $scope.HtmlContent;

                        if (wtt_globals.titleTag != 'No article title used' && $scope.Title != null) {
                            content = '<' + wtt_globals.titleTag + '>' + $scope.Title + '</' + wtt_globals.titleTag + '>' + content;
                        }

                        return content;
                    }

                    function getQualityDetails(details) {
                        if (!details) {
                            return details;
                        }

                        switch (details.Sentiment) {
                            case "1":
                                details.SentimentInfo = "Positive";
                                break;
                            case "0":
                                details.SentimentInfo = "Neutral";
                                break;
                            case "-1":
                                details.SentimentInfo = "Negative";
                                break;
                            default:
                                details.SentimentInfo = "";
                                break;
                        }

                        return details;
                    }

                    function analyzeContentQuality(ruleSet) {

                        // console.log("contentQualityController -> run Content Suggestions");

                        $scope.analyzing = true;
                        startContentQualityCompute();

                        $scope.cqModel = {
                            content: GetPageContent(),
                            languageCode: $scope.localLanguageCode,
                            qualityLevels: $scope.settings,
                            ruleSet: ruleSet
                        };

                        // get content quality suggestions
                        httpService.postData(WttApiBaseUrl + "contentquality/suggestions", $scope.cqModel).then(function(response) {
                            endContentQualityCompute();

                            $scope.userInfo.Credits = $scope.userInfo.Credits - 1;

                            renderSuggestions(response.Suggestions);

                            //Save content quality suggestions to wp database with ajax call
                            saveContentQualitySuggestions(response); //$j.extend({}, response.Details, response.Suggestions)

                            // update run date
                            $scope.LastQualityRun = response.ModifiedDate;
                            $scope.LastModified = response.ModifiedDate;

                            // save details
                            $scope.contentQualityDetails = getQualityDetails(response.Details);

                        }, function error(errorKey) {
                            toastr.error($scope.data.Resources[errorKey].toString());

                            // end analyze animations
                            showError($scope.data.Resources[errorKey].toString());
                            $scope.analyzing = false;
                        });

                    }

                    var saveContentQualitySuggestions = function(suggestions) {

                        var details = {
                            "Details": {
                                "ReadingLevel": suggestions.Details.ReadingLevel,
                                "ReadingTime": suggestions.Details.ReadingTime,
                                "ReadingValues": suggestions.Details.ReadingValues,
                                "Sentiment": suggestions.Details.Sentiment
                            }
                        };

                        var data = $j.extend({}, details, suggestions.Suggestions);
                        wtt_globals.wtt_content_quality_suggestions.value = JSON.stringify(data);
                    };

                    var saveContentQualitySettings = function(settings) {
                        wtt_globals.wtt_content_quality_settings.value = JSON.stringify(settings);
                    };


                    function renderSuggestions(response, broadcast) {
                        $scope.contentQualitySuggestions = response.Suggestions;
                        $scope.QualityScore = response.PageScore;
                        $scope.QualityScoreTag = response.PageScoreTag;
                        $scope.analyzing = false;

                        if (broadcast === false) {
                            return;
                        }
                        $scope.$broadcast('runSuggestions', { engine: "content" });
                    }

                    function startContentQualityCompute() {
                        $scope.QualityScore = ".";
                        $scope.QualityScoreTag = "Analyzing...";
                        $scope.loadingStep = 0;
                        $scope.$broadcast('runSuggestions', { engine: "loading" });
                        $scope.contentQualityLoading = true;
                    }

                    function endContentQualityCompute() {
                        $scope.contentQualityLoading = false;
                    }

                    function showError(error) {
                        $scope.contentQualityLoading = false;
                        $scope.$broadcast('runSuggestions', { engine: "error", error: error });
                    }

                    var escapeHtml = function(text) {
                        var map = {
                            '&': '&amp;',
                            '<': '&lt;',
                            '>': '&gt;',
                            '"': '&quot;',
                            "'": '&#039;'
                        };

                        return text.toString().replace(/[&<>"']/g, function(m) {
                            return map[m];
                        });
                    };

                    $scope.runSuggestions = function() {
                        if ($scope.runRules == true && !$scope.rulesRunning) {

                            $scope.rulesRunning = true;

                            var content = getHtmlContent($scope.HtmlContent, $scope.Title, escapeHtml($scope.Description), $scope.permalink, $scope.Keyword, $scope.checkboxModel.isChecked);

                            $scope.model = {
                                content: content,
                                keywords: $scope.Keyword,
                                languageCode: $scope.localLanguageCode,
                                domain: $scope.domainUrl,
                                synonyms: $scope.pageKeywordSynonyms,
                                ruleSet: $scope.ruleSet
                            };

                            httpService.postData(WttApiBaseUrl + "page/suggestions", $scope.model).then(function(response) {
                                if (!suggestionsRanFirstTime) {
                                    $scope.suggestions = response.Suggestions;
                                    suggestionsRanFirstTime = true;
                                } else {
                                    var updatedSuggestions = response.Suggestions;
                                    _.each($scope.suggestions, function(suggestion, index) {
                                        suggestion.Tag = updatedSuggestions[index].Tag;
                                        suggestion.Importance = updatedSuggestions[index].Importance;
                                        suggestion.Penalty = updatedSuggestions[index].Penalty;
                                        suggestion.Rules = updatedSuggestions[index].Rules;
                                        suggestion.Score = updatedSuggestions[index].Score;
                                        suggestionsService.computeDisplayType(suggestion);
                                    });
                                }

                                $scope.Score = response.PageScore;
                                $scope.ScoreTag = response.PageScoreTag;
                                $scope.runRules = false;
                                $scope.rulesRunning = false;
                                $scope.updateSuggestionsStart();

                                $timeout(function() {
                                    broadcastSelectedNodes();
                                });

                                $scope.$broadcast('runSuggestions', { engine: $scope.activeEngine });
                            });
                        }
                    };

                    $scope.selectEngine = function(engine) {
                        $scope.activeEngine = engine;
                        $scope.$broadcast('runSuggestions', { engine: engine });
                    };

                    $scope.updateSuggestions = function() {
                        $scope.runRules = true;
                    };

                    $scope.$on("runSuggestions", function(event, args) {
                        // console.log(args);

                        $scope.loading = false;
                        $scope.showScore = true;
                        $scope.showError = false;
                        $scope.seoScoreTag = $scope.ScoreTag;
                        $scope.contentScoreTag = $scope.QualityScoreTag;
                        $scope.seoScore = (Math.round($scope.Score || 0)) + "%"; //$scope.Score || 0;
                        $scope.contentScore = (Math.round($scope.QualityScore || 0)) + "%"; //$scope.QualityScore || 0;

                        if (args.engine == "seo") {
                            $scope.seoClass = "page-score";
                            $scope.contentClass = "gray-score";
                            $timeout(function() {
                                $scope.seoPageScoreData = buildActiveChart(parseInt($scope.seoScore));
                                $scope.contentPageScoreData = buildInactiveChart(parseInt($scope.contentScore));
                            }, 0);
                            wtt_globals.wtt_page_score.value = Math.round($scope.Score);
                        } else if (args.engine == "content") {
                            $scope.seoClass = "gray-score";
                            $scope.contentClass = "page-score";
                            $timeout(function() {
                                $scope.seoPageScoreData = buildInactiveChart(parseInt($scope.seoScore));
                                $scope.contentPageScoreData = buildActiveChart(parseInt($scope.contentScore));
                            }, 0);
                        } else if (args.engine == "loading") {
                            $scope.showScore = false;
                            $scope.loading = true;
                            $scope.scoreTag = ".";
                            $scope.primaryScore = "Analyzing...";
                            $scope.secondaryScore = ".";
                        } else if (args.engine == "error") {
                            $scope.scoreTag = ".";
                            $scope.primaryScore = "Analyzing...";
                            $scope.secondaryScore = ".";
                            $scope.error = args.error;
                            $scope.showError = true;
                            $scope.showScore = false;
                            $scope.loading = false;
                        }
                    });

                    /**
                     * Page section/node highlighting
                     */

                    var broadcastSelectedNodes = function() {
                        $scope.$broadcast('editPageController:selectNodes', selectedPageNodes);
                    };

                    var broadcastZeroNodes = function() {
                        var broadcastZeroDeferred = $q.defer();
                        selectedPageNodes = [];
                        $scope.$broadcast('editPageController:selectNodes', selectedPageNodes);
                        broadcastZeroDeferred.resolve();
                        return broadcastZeroDeferred.promise;
                    };

                    $j('#' + wtt_globals.titleFieldName).focus(function() {
                        broadcastZeroNodes().then(function() {
                            selectedPageNodes = ['TITLE'];
                            broadcastSelectedNodes();
                        });
                    });

                    $j('#' + wtt_globals.titleFieldName).blur(function() {
                        broadcastZeroNodes();
                    });

                    $j('#' + wtt_globals.metaDescFieldName).focus(function() {
                        broadcastZeroNodes().then(function() {
                            selectedPageNodes = ['DESCRIPTION'];
                            broadcastSelectedNodes();
                        });
                    });

                    $j('#' + wtt_globals.metaDescFieldName).blur(function() {
                        broadcastZeroNodes();
                    });

                    function hasCredits() {
                        return userInfo.Credits > 0;
                    }

                    $scope.pageHasKeyword = function() {
                        return getContent('wtt-keyword').length != 0;
                    };

                    var inputKeyword = getContent('wtt-keyword');

                    if (inputKeyword.length !== 0) {
                        $scope.useMyKeyword();
                    }

                    //get the snippet title and description for google snippet preview
                    var $element = $j('#' + wtt_globals.titleFieldName),
                        inputTitle = $element.val(),
                        $textElement = $j('#' + wtt_globals.metaDescFieldName),
                        textContent = $textElement.val();

                    setInterval(function() {
                        var currentTitle = $element.val();
                        var currentDesc = $textElement.val();

                        if (currentTitle !== inputTitle) {
                            // A change has happened
                            inputTitle = currentTitle;
                            $scope.Title = inputTitle;

                            if ($scope.Title !== null) {
                                $scope.updateSuggestions();
                            }
                        }

                        if (currentDesc !== textContent) {
                            // A change has happened
                            textContent = currentDesc;
                            $scope.Description = textContent;

                            if ($scope.Description !== null) {
                                $scope.updateSuggestions();
                            }
                        }
                    }, 1000);

                    $scope.useKeyword = function(keyword) {
                        if ($scope.pageHasKeyword()) {
                            document.getElementById('wtt-keyword').setAttribute('value', keyword.Keyword);

                            $scope.Keyword = keyword.Keyword;
                            $scope.Title = $j('#' + wtt_globals.titleFieldName).val();
                            $scope.Description = $j('#' + wtt_globals.metaDescFieldName).val();

                            if (tinymce.activeEditor) {
                                $scope.HtmlContent = tinymce.activeEditor.getContent();
                            } else {
                                $scope.HtmlContent = '';
                            }

                            $scope.runRules = true;
                            $scope.runSuggestions();

                            $timeout(function() {
                                $scope.isCollapsed = true;
                                $scope.keywordSet = true;
                            }, 500);
                        } else {
                            //inputKeywordDB.setAttribute('value', "");
                        }
                    };

                    /**
                     *  Select Keyword Controller
                     */
                    var keywordSourceLangCookie = $cookies.get('wtt_keyword_source_lang');

                    $scope.fillKeyword = function(keyword) {
                        $j("#wtt-keyword").prop("value", keyword.Keyword);
                        $timeout(function() {
                            $scope.submitSearchKeyword();
                        }, 500);
                    };

                    $scope.onKeywordBlur = function() {
                        $j('#wtt-keyword').prop('value', $j.trim(cleanKeyword($j('#wtt-keyword').val())));

                    };

                    $scope.searchModel = {
                        "inputKeyword": "",
                        "KeywordSources": [],
                        "selectedSource": {},
                        "SearchResults": []
                    };

                    var keywordSourceCodeMap = {
                        en: 'us'
                    };

                    // Get Google languages
                    function loadKeywordSources() {
                        keywordService.getKeywordSources().then(
                            function loaded(keywordSources) {
                                $scope.KeywordSources = keywordSources;

                                var appLanguageCode = languageService.getActiveLanguageCode();
                                var mappedLanguageCode = keywordSourceCodeMap[appLanguageCode] || appLanguageCode;

                                var appLanguageKeywordSource = _.find(keywordSources, function(item) {
                                    return item.Value == mappedLanguageCode;
                                });

                                var selectedKeywordSource = _.isUndefined(keywordSourceLangCookie) ? appLanguageKeywordSource : _.find(keywordSources, function(item) {
                                    return item.Value == keywordSourceLangCookie;
                                });

                                $scope.searchModel.selectedSource = selectedKeywordSource;
                            });
                    }

                    $scope.selectKeywordSource = function(keywordSource) {
                        $scope.searchModel.selectedSource = keywordSource;
                        $cookies.put('wtt_keyword_source_lang', keywordSource.Value);
                    };

                    // Mapping Keyword Scores
                    var mapKeywordScores = function(keywords) {
                        var overallScoreLabelMap = {
                            0: "Very Poor",
                            1: "Moderate",
                            2: "Good"
                        };

                        var overallScoreHelpMap = {
                            0: "Overall Score Very Poor",
                            1: "Overall Score Moderate",
                            2: "Overall Score Good"
                        };

                        var overallScoreClassMap = {
                            0: "interval-very-poor",
                            1: "interval-moderate",
                            2: "interval-good"
                        };

                        var overallScoreLabelClassMap = {
                            0: "label-danger",
                            1: "label-warning",
                            2: "label-success"
                        };

                        var volumeScoreMap = {
                            0: {
                                className: "very-low",
                                label: "Very Low",
                                helpText: "Monthly search volume on this keyword is very low."
                            },
                            1: {
                                className: "low",
                                label: "Low",
                                helpText: "Monthly search volume on this keyword is low."
                            },
                            2: {
                                className: "moderate",
                                label: "Moderate",
                                helpText: "Monthly search volume on this keyword is moderate."
                            },
                            3: {
                                className: "high",
                                label: "High",
                                helpText: "Monthly search volume on this keyword is high. A lot of searches are done on this keyword."
                            },
                            4: {
                                className: "very-high",
                                label: "Very High",
                                helpText: "Monthly search volume on this keyword is very high. A lot of search are done on this keyword."
                            }
                        };

                        var competitionMap = {
                            4: {
                                className: "very-easy",
                                label: "Very Easy",
                                helpText: "This means that competition is low on this keyword. Not a lot of (strong) sites are also optimised on this keyword."
                            },
                            3: {
                                className: "easy",
                                label: "Easy",
                                helpText: "Competition on this keyword is not very strong. Not a lot of (strong) sites and pages are also optimised on this keyword."
                            },
                            2: {
                                className: "moderate",
                                label: "Moderate",
                                helpText: "Competition on this keyword is moderate. Quite some (strong) sites and pages are also optimised on this keyword."
                            },
                            1: {
                                className: "hard",
                                label: "Hard",
                                helpText: "Competition on this keyword is strong. A lot of (strong) sites and pages are also optimised on this keyword."
                            },
                            0: {
                                className: "very-hard",
                                label: "Very Hard",
                                helpText: "Competition on this keyword is very strong. A lot of (strong) sites and pages are also optimised on this keyword."
                            }
                        };

                        _.each(keywords, function(result) {
                            if (result.OverallScore >= 0) {
                                result.OverallScoreClass = overallScoreClassMap[result.OverallScore];
                                result.OverallScoreHelp = overallScoreHelpMap[result.OverallScore];
                                result.OverallScoreLabelClass = overallScoreLabelClassMap[result.OverallScore];
                                result.OverallScoreLabel = overallScoreLabelMap[result.OverallScore];
                            } else {
                                result.OverallScoreClass = "interval-moderate";
                                result.OverallScoreLabel = "N/A";
                                result.OverallScoreHelp = "This is an estimation because we have limited data on keyword volume and/or competition";
                                result.OverallScoreLabelClass = "label-warning";
                            }

                            if (result.VolumeScore >= 0) {
                                result.SearchVolumeScoreAttrs = volumeScoreMap[result.VolumeScore];
                            } else {
                                result.SearchVolumeScoreAttrs = {
                                    className: "moderate",
                                    label: "N/A",
                                    helpText: "This is a rough estimation because we have limited data on volume for this keyword."
                                };
                            }

                            if (result.CompetitionScore >= 0) {
                                result.CompetitionScoreAttrs = competitionMap[result.CompetitionScore];
                            } else {
                                result.CompetitionScoreAttrs = {
                                    className: "moderate",
                                    label: "N/A",
                                    helpText: "This is a rough estimation because we have limited data on competition for this keyword."
                                };
                            }
                        });
                        return keywords;
                    };

                    $scope.displaySearchVolume = function(volume) {
                        if (volume < 50) {
                            return "< 50";
                        }
                        return volume;
                    };

                    $scope.submitSearchKeyword = function() {
                        $scope.errorMsg = null;

                        if (hasCredits()) {

                            if (!validateKeyword()) {
                                toastr.warning($scope.errorMsg, {
                                    closeButton: true,
                                    timeOut: 3000
                                });
                                return;
                            }

                            $scope.searchPromise = keywordService.searchKeyword($j.trim(cleanKeyword(removeSpecialChars(getContent("wtt-keyword")))), $scope.searchModel.selectedSource.Value, $scope.localLanguageCode, 10);
                            $scope.searchPromise.then(
                                function loaded(results) {
                                    $timeout(function() {
                                        $j('body').animate({
                                            scrollTop: $j("#score-analysis-area").offset().top
                                        }, 1000);
                                    });

                                    $scope.searchModel.SearchResults = mapKeywordScores(results);

                                    $scope.inputKeywordResult = _.find(results, function(item) {
                                        return item.Selected == true;
                                    });

                                },
                                function(message) {
                                    toastr.warning("You have used all your available keyword search credits.", message);
                                }
                            );
                        } else {
                            toastr.warning("You have used all your available keyword search credits.", "Credits below limit.");
                        }
                    };

                    function validateKeyword() {
                        if (getContent('wtt-keyword') === "" || getContent('wtt-keyword') == null) {
                            $scope.errorMsg = "Keyword is required!";
                            return false;
                        }

                        if (getContent('wtt-keyword').length > 250) {
                            $scope.errorMsg = "Keyword is invalid!";
                            return false;
                        }

                        if (getContent('wtt-keyword').indexOf(",") > -1 || getContent('wtt-keyword').indexOf(";") > -1) {
                            $scope.errorMsg = "We accept only one keyword. Please do not use separator special chars , or ; ";
                            return false;
                        }

                        return true;
                    }

                    function cleanKeyword(keyword) {
                        if (isNullOrEmpty(keyword)) {
                            return keyword;
                        }
                        return cleanMeta(keyword, false);
                    }

                    // replace with space the special chars and escape single quotes
                    function cleanMeta(text, escape) {
                        // replace with space the special chars
                        var cleanText = (text ? text : "").replace(/["=<>\{\}\[\]\\\/]/gi, ' ');
                        // escape single quotes
                        if (escape == true) {
                            cleanText = cleanText.replace(/[']/g, '\\$&').replace(/\u0000/g, '\\0');
                        }
                        return cleanText;
                    }

                    $scope.dynamicPopover = {
                        templateUrl: 'moreinfo.html'
                    };

                    var initialCQTemplate = {
                        "PageScore": 0.0,
                        "Suggestions": [{
                            "Tag": "Readability",
                            "MetaTag": "Readability",
                            "Rules": null,
                            "Importance": 70.0,
                            "Score": 0.0,
                            "Penalty": 0.0,
                            "Tooltip": null,
                            "SortIndex": 1,
                            "Metadata": {
                                "Category": 20,
                                "ShortName": "Readability",
                                "DisplayName": "Readability",
                                "DisplayType": null,
                                "Order": 1,
                                "Settings": [{ "DisplayName": "Elementary", "Value": "1" }, {
                                    "DisplayName": "Highschool",
                                    "Value": "2"
                                }, { "DisplayName": "University", "Value": "3" }],
                                "Rules": [{
                                    "Name": "ReadingLevel",
                                    "Value": "1",
                                    "IsPrimary": true,
                                    "IsStringValue": false
                                }, {
                                    "Name": "DifficultWordsLevel",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }, {
                                    "Name": "LongSentencesLevel",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }, {
                                    "Name": "SentenceLengthLevel",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }, {
                                    "Name": "PassiveVoice",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }, {
                                    "Name": "DifficultWordsLevel",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }]
                            }
                        }, {
                            "Tag": "Adjectives",
                            "MetaTag": "Adjectives",
                            "Rules": null,
                            "Importance": 20.0,
                            "Score": 0.0,
                            "Penalty": 0.0,
                            "Tooltip": null,
                            "SortIndex": 1,
                            "Metadata": {
                                "Category": 21,
                                "ShortName": "Adjectives",
                                "DisplayName": "Adjectives",
                                "DisplayType": null,
                                "Order": 2,
                                "Settings": null,
                                "Rules": [{
                                    "Name": "AdjectivesLevel",
                                    "Value": "1",
                                    "IsPrimary": true,
                                    "IsStringValue": false
                                }]
                            }
                        }, {
                            "Tag": "Whitespaces",
                            "MetaTag": "Whitespaces",
                            "Rules": null,
                            "Importance": 20.0,
                            "Score": 0.0,
                            "Penalty": 0.0,
                            "Tooltip": null,
                            "SortIndex": 1,
                            "Metadata": {
                                "Category": 23,
                                "ShortName": "Whitespaces",
                                "DisplayName": "Whitespaces",
                                "DisplayType": null,
                                "Order": 5,
                                "Settings": null,
                                "Rules": [{
                                    "Name": "WhitespacesLevel",
                                    "Value": "1",
                                    "IsPrimary": true,
                                    "IsStringValue": false
                                }, {
                                    "Name": "BulletPointsLevel",
                                    "Value": "1",
                                    "IsPrimary": false,
                                    "IsStringValue": false
                                }]
                            }
                        }, {
                            "Tag": "Gender",
                            "MetaTag": "Gender",
                            "Rules": null,
                            "Importance": 15.0,
                            "Score": 0.0,
                            "Penalty": 0.0,
                            "Tooltip": null,
                            "SortIndex": 1,
                            "Metadata": {
                                "Category": 22,
                                "ShortName": "Gender",
                                "DisplayName": "Gender",
                                "DisplayType": null,
                                "Order": 3,
                                "Settings": [{ "DisplayName": "Female", "Value": "f" }, {
                                    "DisplayName": "Neutral",
                                    "Value": "n"
                                }, { "DisplayName": "Male", "Value": "m" }],
                                "Rules": [{
                                    "Name": "GenderLevel",
                                    "Value": "n",
                                    "IsPrimary": true,
                                    "IsStringValue": true
                                }]
                            }
                        }, {
                            "Tag": "Sentiment",
                            "MetaTag": "Sentiment",
                            "Rules": null,
                            "Importance": 10.0,
                            "Score": 0.0,
                            "Penalty": 0.0,
                            "Tooltip": null,
                            "SortIndex": 1,
                            "Metadata": {
                                "Category": 30,
                                "ShortName": "Sentiment",
                                "DisplayName": "Sentiment",
                                "DisplayType": null,
                                "Order": 4,
                                "Settings": [{
                                    "DisplayName": "Negative",
                                    "Value": "negative"
                                }, { "DisplayName": "Neutral", "Value": "neutral" }, {
                                    "DisplayName": "Positive",
                                    "Value": "positive"
                                }],
                                "Rules": [{
                                    "Name": "SentimentLevel",
                                    "Value": "positive",
                                    "IsPrimary": true,
                                    "IsStringValue": true
                                }]
                            }
                        }],
                        "PageScoreTag": "Let’s get started!",
                        "RuleSet": "ContentQuality"
                    };


                    function init() {

                        loadKeywordSources();
                        loadLanguages();

                        $scope.QualityLevels = '';

                        // get content quality settings from wp
                        if (wtt_globals.getCQSettings !== "") {
                            $scope.QualityLevels = wtt_globals.getCQSettings;
                        }

                        // get page quality levels from settings
                        if ($scope.QualityLevels) {
                            $scope.settings = $scope.QualityLevels;
                        } else {
                            $scope.settings = getDefaultQualitySettings();
                        }
                        if (wtt_globals.getLastSuggestions !== "") {
                            // load last run suggestions
                            $scope.contentQualityDetails = getQualityDetails(wtt_globals.getLastSuggestions.Details);

                            var lastRunSuggestions = wtt_globals.getLastSuggestions;
                            //map the new Metadata template to the old Suggestions array (to prevent errors)
                            if (typeof wtt_globals.getLastSuggestions.Suggestions[0].Metadata == "undefined") {
                                var contentQualitySuggestions = _.map(initialCQTemplate.Suggestions, function(element) {
                                    var treasure = _.findWhere(wtt_globals.getLastSuggestions.Suggestions, { Tag: element.Tag });
                                    return _.extend(element, treasure);
                                });

                                lastRunSuggestions.Suggestions = contentQualitySuggestions;
                            }
                            renderSuggestions(lastRunSuggestions, false);
                        } else {
                            // load initial template
                            $scope.contentQualitySuggestions = initialCQTemplate.Suggestions; //new template does not have extra Suggestions object
                            $scope.QualityScoreTag = initialCQTemplate.Suggestions.PageScoreTag;
                        }

                        var languageCodeDB = document.getElementById('wtt-language-code-field');
                        languageCodeDB.setAttribute('value', $scope.localLanguageCode);
                        if ($scope.pageHasKeyword()) {
                            $scope.keywordSet = true;
                        } else {
                            $j("#wtt-keyword").focus();
                        }
                    }

                    init();

                    // Language service
                    function loadLanguages() {
                        languageService.getLanguages().then(
                            function loaded(languages) {
                                $scope.languages = languages;
                            });
                    }

                    $scope.activeLanguageCode = languageService.getActiveLanguageCode();

                    $scope.$on('userLanguageChanged', function() {
                        $scope.activeLanguageCode = languageService.getActiveLanguageCode();
                    });

                    $scope.setActiveLanguage = function(language) {
                        $scope.activeLanguageCode = language.LanguageCode;
                        updateUserLanguage(language.LanguageCode);
                    };

                    function updateUserLanguage(languageCode) {
                        httpService.postData(WttApiBaseUrl + "user/language/" + languageCode).then(function() {
                            $scope.localLanguageCode = languageCode;
                            var languageCodeDB = document.getElementById('wtt-language-code-field');
                            languageCodeDB.setAttribute('value', languageCode);
                            $scope.$broadcast('languageChanged');
                            $scope.useMyKeyword();
                            if ($scope.localLanguageCode == wtt_globals.currentLanguage) {
                                wtt_globals.wtt_post_languagecode.value = '';
                            } else {
                                wtt_globals.wtt_post_languagecode.value = $scope.localLanguageCode;
                            }
                        });
                    }

                });
            } else {
                if (wtt_globals.pageScore !== "") {
                    $scope.seoScore = wtt_globals.pageScore;
                }
            }
        });
    }
]);

app.factory("suggestionsService", function() {
    var computeDisplayType = function(suggestion) {
        suggestion.displayScore = suggestion.Score == 0 ? suggestion.Importance * 0.2 : suggestion.Score;
        suggestion.progressType = suggestion.Score < suggestion.Importance / 3 ? "danger" :
            suggestion.Score < suggestion.Importance ? "warning" : "success";
    };

    return {
        computeDisplayType: computeDisplayType
    };
});

app.factory("synonymService", ['$http', '$q', 'httpService',
    function($http, $q, httpService) {
        var WttApiBaseUrl = wtt_globals.wtt_base_api_url;

        var getSynonyms = function(word) {
            var dataUrl = WttApiBaseUrl + "synonyms/" + encodeURIComponent(word);
            var deffered = $q.defer();
            httpService.getData(dataUrl).then(function(synonyms) {
                deffered.resolve(synonyms);
            });
            return deffered.promise;
        };

        return {
            getSynonyms: getSynonyms
        };
    }
]);

app.factory("languageService", ['$http', '$q', 'httpService', '$cookies',
    function($http, $q, httpService, $cookies) {
        var WttApiBaseUrl = wtt_globals.wtt_base_api_url;

        var getLanguages = function() {
            return httpService.getData(WttApiBaseUrl + "languages");
        };

        var getActiveLanguageCode = function() {

            if (!_.isUndefined($cookies.get('wtt_lang'))) {
                return $cookies.get('wtt_lang');
            }

            var language = window.navigator.userLanguage || window.navigator.language;
            if (language.indexOf("nl") >= 0) {
                return "nl";
            }
            if (language.indexOf("de") >= 0) {
                return "de";
            }
            return "en";
        };

        return {
            getLanguages: getLanguages,
            getActiveLanguageCode: getActiveLanguageCode
        };
    }
]);

app.directive('enforceMaxTags', function() {
    return {
        require: 'ngModel',
        link: function(scope, element, attrs, ngModelController) {
            var maxTags = attrs.maxTags ? parseInt(attrs.maxTags, "10") : null;

            ngModelController.$validators.checkLength = function(value) {
                if (value && maxTags && value.length > maxTags) {
                    value.splice(value.length - 1, 1);
                }
                return value;
            };
        }
    };
});

app.directive("wttContentQuality", function() {
    return {
        template: '<div ng-include="getTemplateUrl()"></div>',
        link: function(scope) {
            scope.getTemplateUrl = function() {
                return wtt_globals.pluginsUrl + 'inc/directives/wtt-content-quality.html';
            };

        }
    };
});

app.directive('wttSuggestion', ["suggestionsService", "$sce", "stateService",
    function(suggestionsService, $sce, stateService) {
        return {
            template: '<div ng-include="getTemplateUrl()"></div>',
            scope: {
                suggestion: '=',
                uid: '=',
                index: '@'
            },
            replace: true,
            link: function(scope) {
                scope.suggestion.selected = false;
                scope.suggestion.isCollapsed = true;
                suggestionsService.computeDisplayType(scope.suggestion);

                scope.getTemplateUrl = function() {
                    return wtt_globals.pluginsUrl + 'inc/directives/wtt-suggestion.html';
                };

                var data = stateService.data;
                scope.data = data;

                var tagMap = {
                    'Page Title': ['TITLE'],
                    'Page Description': ['DESCRIPTION'],
                    'Headings': ['HEADINGS'],
                    'Main Content': ['MAINCONTENT'],
                    'Miscellaneous': ['MISCELLANEOUS']
                };

                var suggestionTags = tagMap[scope.suggestion.Tag];

                scope.$on('editPageController:selectNodes', function(event, selectedNodeNames) {
                    if (selectedNodeNames.length === 0) {
                        scope.suggestion.selected = false;
                        if (scope.suggestion.Score == scope.suggestion.Importance) {
                            scope.suggestion.isCollapsed = true;
                        }
                    } else {

                        // Title or description collapse behaviour
                        if (_.contains(selectedNodeNames, 'TITLE') || _.contains(selectedNodeNames, 'DESCRIPTION')) {
                            scope.suggestion.isCollapsed = true;
                            if (_.contains(selectedNodeNames, 'TITLE') && scope.suggestion.Tag == 'Page Title' && scope.suggestion.Score !== scope.suggestion.Importance) {
                                scope.suggestion.isCollapsed = false;
                            } else if (_.contains(selectedNodeNames, 'DESCRIPTION') && scope.suggestion.Tag == 'Page Description' && scope.suggestion.Score !== scope.suggestion.Importance) {
                                scope.suggestion.isCollapsed = false;
                            }
                        } else {

                            // Body nodes collapse behaviour
                            if (scope.suggestion.Tag == 'Page Title' || scope.suggestion.Tag == 'Page Description') {
                                scope.suggestion.isCollapsed = true;
                            } else if (scope.suggestion.Score !== scope.suggestion.Importance) {
                                scope.suggestion.isCollapsed = false;
                            }
                        }

                        // Selection behaviour
                        _.each(suggestionTags, function(suggestionTag) {
                            if (_.contains(selectedNodeNames, suggestionTag)) {
                                scope.suggestion.selected = true;
                                scope.suggestion.isCollapsed = false;
                                return;
                            }
                        });
                    }
                });

                function camelize(str) {
                    return str.replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index) {
                        if (+match === 0) return "";
                        return index == 0 ? match.toLowerCase() : match.toUpperCase();
                    });
                }

                data.Resources.forEach(function(el, i) {
                    data.Resources[el.ResourceKey] = $sce.trustAsHtml(el.HtmlContent);
                });

                var setDisplayTexts = function() {
                    var resourceName = scope.suggestion.Tag.replace(new RegExp(" ", "g"), "") + "Label";
                    scope.displayName = data.Resources[resourceName].toString();

                    scope.name = scope.suggestion.Tag.replace(new RegExp(" ", 'g'), "") + 'Suggestion';
                    scope.tip = data.Resources[scope.name];

                    scope.domId = camelize(scope.suggestion.Tag) + 'SuggestionBox';
                };

                setDisplayTexts();
            }
        };
    }
]);

app.directive("wttSuggestionContentQuality", ["suggestionsService", "$sce", "stateService",
    function(suggestionsService, $sce, stateService) {
        return {
            template: '<div ng-include="getTemplateUrl()"></div>',
            scope: {
                settings: "=",
                suggestion: "=",
                uid: "=",
                index: "@",
            },
            replace: true,
            link: function(scope, elem, attr) {
                scope.getTemplateUrl = function() {
                    return wtt_globals.pluginsUrl + 'inc/directives/wtt-suggestion-content-quality.html';
                };

                var updateOnAction = function(value) {
                    scope.suggestion.selected = (value != 0);
                };

                var data = stateService.data;
                data.viewExtraInfoToggle = {};
                scope.sliderInfo = {};
                scope.data = data;

                scope.setRuleProp = function(rules, value) {
                    for (var i = 0; i < rules.length; i++) {
                        var rule = rules[i];
                        var prop = rule.Name;
                        if (value === 0) {
                            if (scope.settings[prop]) {
                                scope.settings[prop] = rule.IsStringValue ? '' : 0;
                            } else {
                                scope.settings[prop] = rule.Value;
                            }
                        } else {
                            scope.settings[prop] = value;
                        }
                    }
                };

                var buildAction = function(metadata) {
                    // console.log(metadata);

                    var primaryRule = metadata.Rules[0];
                    var prop = primaryRule.Name;
                    var buttonList = _.map(metadata.Settings, function(item) {
                        return {
                            label: item.DisplayName,
                            tip: item.DisplayName,
                            value: item.Value
                        };
                    });

                    return {
                        buttons: buttonList,
                        action: function(button) {
                            scope.setRuleProp(metadata.Rules, button.value);
                            updateOnAction(scope.settings[prop]);
                        },
                        selected: function(button) {
                            return scope.settings[prop] == button.value;
                        },
                        active: !!scope.settings[prop],
                        display: metadata.DisplayType
                    };
                };

                var action = buildAction(scope.suggestion.Metadata);
                scope.actionConfig = action;

                scope.suggestion.selected = scope.suggestion.Importance > 0;
                scope.suggestion.isCollapsed = false;

                suggestionsService.computeDisplayType(scope.suggestion);

                var camelize = function(str) {
                    return str.replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index) {
                        if (+match === 0) return "";
                        return index == 0 ? match.toLowerCase() : match.toUpperCase();
                    });
                };

                data.Resources.forEach(function(el, i) {
                    data.Resources[el.ResourceKey] = $sce.trustAsHtml(el.HtmlContent);
                });

                var setDisplayTexts = function() {
                    var resourceName = scope.suggestion.Tag.replace(new RegExp(" ", "g"), "") + "Label";
                    scope.displayName = data.Resources[resourceName].toString();

                    scope.name = scope.suggestion.Tag.replace(new RegExp(" ", 'g'), "") + 'Suggestion';
                    scope.tip = data.Resources[scope.name];

                    scope.domId = camelize(scope.suggestion.Tag) + 'SuggestionBox';
                };

                setDisplayTexts();

                scope.$on("languageChanged", setDisplayTexts);

                scope.viewExtraInfoList = function(rule) {
                    if (scope.data.viewExtraInfoToggle[rule.Text] == null || scope.data.viewExtraInfoToggle[rule.Text] === true) {
                        Object.keys(scope.data.viewExtraInfoToggle).forEach(function(ruleText, index) {
                            scope.data.viewExtraInfoToggle[ruleText] = true;
                        });

                        scope.data.viewExtraInfoToggle[rule.Text] = false;
                    } else {
                        scope.data.viewExtraInfoToggle[rule.Text] = true;
                    }

                    if (scope.sliderInfo[rule.Text] != null) {
                        return;
                    }

                    scope.processSliderInfo(rule);
                };

                scope.processTags = function(localSliderInfo) {
                    return _.map(localSliderInfo.List, function(item) {
                        return {
                            word: item.word,
                            count: item.count,
                            type: item.type || 'blue',
                            dataSource: item.to && item.to.length > 0 ? item.to : null,
                            tip: item.to ? item.to.map(function(ds) { return ds.word; }).join(", ") : '',
                            suppressIgnore: localSliderInfo.SuppressIgnore
                        };
                    });
                };

                scope.processSliderInfo = function(rule) {
                    scope.sliderInfo[rule.Text] = JSON.parse(rule.ExtraInfo);
                    var localSliderInfo = scope.sliderInfo[rule.Text];
                    localSliderInfo.tags = localSliderInfo.List;

                    if (localSliderInfo.Type == "info") {
                        localSliderInfo.tags = scope.processTags(localSliderInfo);
                        return localSliderInfo;
                    }

                    if (localSliderInfo.Type == "list") {
                        localSliderInfo.tags = _.chain(localSliderInfo.List).groupBy(function(item) {
                            return item;
                        }).map(function(items, key) {
                            return {
                                word: key,
                                count: items.length
                            };
                        }).value();
                    } else if (localSliderInfo.Type == "fullList") {
                        localSliderInfo.tags = _.chain(localSliderInfo.List).groupBy(function(item) {
                            return item.Word;
                        }).map(function(items, key) {
                            return {
                                word: key,
                                count: items.length,
                                type: items.length > 0 ? items[0].Type : '',
                                dataSource: items[0].To ? items[0].To.map(function(ds) { return ds.word; }).join(", ") : ''
                            };
                        }).value();
                    } else if (localSliderInfo.Type == "listex") {
                        localSliderInfo.tags = _.chain(localSliderInfo.List).map(function(item) {
                            return {
                                word: item.Name,
                                count: item.Count,
                                type: item.Type || '',
                                dataSource: item.To || ''
                            };
                        }).value();
                    }

                    return localSliderInfo;
                };
            }
        };
    }
]);

app.directive("wttPageSlideout", function() {
    return {
        template: '<div ng-include="getTemplateUrl()"></div>',
        scope: {
            info: "=",
        },
        replace: true,
        link: function(scope, element) {
            scope.getTemplateUrl = function() {
                return wtt_globals.pluginsUrl + 'inc/directives/wtt-page-slideout.html';
            };
        }
    };
});

app.factory("keywordService", ['$http', '$q', '$rootScope', 'httpService',
    function($http, $q, $rootScope, httpService) {
        var WttApiBaseUrl = wtt_globals.wtt_base_api_url + 'keywords';

        var getKeywordSources = function() {
            return httpService.getData(WttApiBaseUrl + "/sources");
        };

        var searchKeyword = function(keyword, database, language) {
            var deffered = $q.defer();

            httpService.getData(WttApiBaseUrl + "/" + keyword + "/" + database + "/" + language)
                .then(function(results) {
                    deffered.resolve(results);
                }, function(results) {
                    return deffered.reject(results);
                });

            return deffered.promise;
        };

        return {
            getKeywordSources: getKeywordSources,
            searchKeyword: searchKeyword
        };
    }
]);

app.factory("httpService", ['$http', '$q', 'toastr',
    function($http, $q, toastr) {

        var getData = function(url) {

            var deffered = $q.defer();

            // Make a get request
            $http.get(url, {
                    withCredentials: true
                })
                .success(function(data, status, headers, config) {
                    deffered.resolve(data);
                }).error(function(error, status) {

                    if (status === 401) {
                        return;
                    }

                    if (status === 500) {
                        toastr.error('Please contact us at <a href="mailto:support@webtexttool.com">support@webtexttool.com</a>', 'Something went wrong while retrieving the data!', {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });
                        return;
                    }

                    deffered.reject(error.Message);
                });

            return deffered.promise;
        };


        var postData = function(url, dataIn) {

            var deffered = $q.defer();

            // Make a post request
            $http.post(url, dataIn, {
                    withCredentials: true
                })
                .success(function(data, status, headers, config) {
                    deffered.resolve(data);
                }).error(function(error, status) {

                    if (status === 400) {
                        toastr.error('Your request params are empty or invalid!', 'Bad Request', {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });
                        return;
                    }

                    if (status === 401) {
                        toastr.error(error.Message, 'Unauthorized', {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });
                        return;
                    }

                    if (status === 403) {
                        toastr.error('Please contact us at <a href="mailto:support@webtexttool.com">support@webtexttool.com</a>', 'You\'ve reached the number of requests limit!', {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });
                        return;
                    }

                    if (status === 500) {
                        toastr.error('Please contact us at <a href="mailto:support@webtexttool.com">support@webtexttool.com</a>', 'Something went wrong!', {
                            closeButton: true,
                            allowHtml: true,
                            timeOut: 0,
                            extendedTimeOut: 0
                        });

                        deffered.reject(error.Message);
                    }

                    deffered.reject(error.Message);
                });

            return deffered.promise;

        };

        return {
            getData: getData,
            postData: postData
        };
    }
]);

app.factory("stateService", [function() {
    var data = {
        Resources: [{
                "ResourceKey": "CQGenericError",
                "HtmlContent": "We’re sorry, we could not analyze your content. Please try again or contact support@webtexttool.com in case the issues persist.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "ContentRequiredError",
                "HtmlContent": "Your page needs some content.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "ContentMinLengthError",
                "HtmlContent": "Your page content must have at least 150 words.",
                "LanguageCode": "en"
            },
            { "ResourceKey": "GenericError", "HtmlContent": "Something went wrong!", "LanguageCode": "en" },
            {
                "ResourceKey": "LanguageNotSupportedError",
                "HtmlContent": "Detected language is not yet supported!",
                "LanguageCode": "en"
            },
            { "ResourceKey": "PageTitleLabel", "HtmlContent": "Page Title", "LanguageCode": "en" },
            { "ResourceKey": "PageDescriptionLabel", "HtmlContent": "Page Description", "LanguageCode": "en" },
            { "ResourceKey": "HeadingsLabel", "HtmlContent": "Headings", "LanguageCode": "en" },
            { "ResourceKey": "MainContentLabel", "HtmlContent": "Main Content", "LanguageCode": "en" },
            { "ResourceKey": "MiscellaneousLabel", "HtmlContent": "Miscellaneous", "LanguageCode": "en" },
            {
                "ResourceKey": "HeadingsSuggestion",
                "HtmlContent": "To optimize your content both in terms of readability and SEO, you should structure your content by adding several headings. At the start of your page you normally have an H1 / heading 1 with the title of your page. In some CMS&#39;s / themes this H1 is added automatically. If this is the case, you can select the option &quot;Process page title as H1&quot; from the settings above. Next to H1, you should add smaller heading (H2-H6) to structure your content even further.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "MainContentSuggestion",
                "HtmlContent": "Here you will find several important suggestions for your content. Please have a look at our knowledgebase (Learn tab in the app) to find more background information about these suggestions.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "MiscellaneousSuggestion",
                "HtmlContent": "Here you will find suggestions to optimize your content. These will have smaller impact on overall optimization, but are good to consider and see if they can fit in your content.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "PageTitleSuggestion",
                "HtmlContent": "<p>The Page title is important to search engines. And therefore it&#39;s important to you. Think of a catchy title that will trigger a user to click on your page when it&#39;s listed in the search results. Of course it should also cover the content of the page.</p>\r\n\t",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "PageDescriptionSuggestion",
                "HtmlContent": "<p>The page description is important because it&#39;s shown in the search results and it will tell the search and the users what your page is about. So provide a good description of your content and make sure you follow the suggestion for creating a perfect description of your page.</p>\r\n\t",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "Heading1Suggestion",
                "HtmlContent": "<p>A H1 / Header section at the beginning of your page is important because it&#39;s the readable introduction of your page. In some CMS&#39;s the Page Title is automatically inserted at the top of a page in H1/Header 1.</p>\r\n\t",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "Heading2to6Suggestion",
                "HtmlContent": "<p>Use smaller headings (h2, h3, h4, h5 and/or h6) in your content to highlight / summarize paragraphs. Using headers will make it easier for you reader to &quot;scan&quot; the contents of your page. It allows you to catch the reader&#39;s attention.</p>\r\n\t",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "BodySuggestion",
                "HtmlContent": "<p>These suggestions are related to overall content on your page. Our rules suggest a minimum number of words for your page. Also related to the length of your content, is the number of times you should use your keywords. This way you can avoid to put your keyword too many times in the content (&quot;keyword stuffing&quot;), but also make sure that you use your keyword enough times so it will be clear for the search engine what the content is about.</p>\r\n\t",
                "LanguageCode": "en"
            },
            { "ResourceKey": "SentimentLabel", "HtmlContent": "Sentiment", "LanguageCode": "en" },
            { "ResourceKey": "PageLabel", "HtmlContent": "Page", "LanguageCode": "en" },
            { "ResourceKey": "ReadabilityLabel", "HtmlContent": "Readability", "LanguageCode": "en" },
            { "ResourceKey": "AdjectivesLabel", "HtmlContent": "Text credibility", "LanguageCode": "en" },
            { "ResourceKey": "GenderLabel", "HtmlContent": "Target audience", "LanguageCode": "en" },
            { "ResourceKey": "WhitespacesLabel", "HtmlContent": "Text layout", "LanguageCode": "en" },
            { "ResourceKey": "BulletpointsLabel", "HtmlContent": "Bulletpoints", "LanguageCode": "en" },
            {
                "ResourceKey": "ReadabilitySuggestion",
                "HtmlContent": "Readability: multiple checks on complexity level of the content (reading score/long sentences/difficult words).",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "AdjectivesSuggestion",
                "HtmlContent": "Checks the use of adjectives in your text. Over- or underuse of adjectives will decrease effectiveness of your text.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "GenderSuggestion",
                "HtmlContent": "Gender check on level (confidence) of content target.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "SentimentSuggestion",
                "HtmlContent": "Set your desired sentiment and see if your content matches. If it doesn't match, it will show you which words to change.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "WhitespacesSuggestion",
                "HtmlContent": "Checks the use of white spaces in your content. Use this to make your content easier to scan and read.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "BulletpointsSuggestion",
                "HtmlContent": "Checks the use of bulletpoints in your content. Use these to make the text easier to scan and read.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "PageSuggestion",
                "HtmlContent": "Your content size is too big.",
                "LanguageCode": "en"
            },
            {
                "ResourceKey": "LanguageLevelHelp",
                "HtmlContent": "<ul>\r\n\t\t<li>\r\n\t\t\t<span>Basic User (A1, A2)</span>\r\n\t\t\t<ul>\r\n\t\t\t\t<li><span>A1 (Beginner)</span></li>\r\n\t\t\t\t<li><span>A2 (Elementary)</span></li>\r\n\t\t\t</ul>\r\n\t\t</li>\r\n\t\t<li>\r\n\t\t\t<span>Independent User (B1, B2)</span>\r\n\t\t\t<ul>\r\n\t\t\t\t<li><span>B1 (Intermediate)</span></li>\r\n\t\t\t\t<li><span>B2 (Upper-Intermediate)</span></li>\r\n\t\t\t</ul>\r\n\t\t</li>\r\n\t\t<li>\r\n\t\t\t<span>Proficient User (C1, C2)</span>\r\n\t\t\t<ul>\r\n\t\t\t\t<li><span>C1 (Advanced)</span></li>\r\n\t\t\t\t<li><span>C2 (Proficiency)</span></li>\r\n\t\t\t</ul>\r\n\t\t</li>\r\n\t</ul>",
                "LanguageCode": "en"
            }
        ],
        loading: false,
        showSpinner: true,
        languages: null,
        sliderInfo: null
    };

    var clear = function() {
        data.languages = null;
        data.sliderInfo = null;
    };

    return {
        data: data,
        clear: clear,
    };
}]);

app.factory('authInterceptor', ["$q",
    function($q) {
        var sessionInjector = {
            request: function(config) {
                if (config.url.indexOf(wtt_globals.wtt_short_url) > 0) {
                    config.headers = config.headers || {};
                    config.headers.Authorization = 'Bearer ' + localStorage.getItem('wtt_token');
                    config.headers.WttSource = 'Joomla';
                }
                return config;
            },
            response: function(response) {
                return response || $q.when(response);
            }
        };
        return sessionInjector;
    }
]);