<?php
/**
 * Webtexttool - SEO made easy, for everyone
 *
 * @author       Jan Martin Roelofs
 * @copyright    (c) Webtexttool - 2019
 * @package      webtexttool
 * @license      http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version      2.0.1
 * @date         2019-04-06
 *
 */

defined('_JEXEC') or die;

/**
 * webtexttool system plugin
 */

require_once 'inc/class-webtexttool.php';

class plgSystemWebtexttool extends JPlugin {

    function onAjaxWebtexttool() {
        JSession::checkToken('get') or die ('Invalid Token');
        $jinput = JFactory::getApplication()->input;

        $titleTag = $jinput->json->get('titleTag');
        if (isset ($titleTag) && in_array($titleTag, Webtexttool::get_allowed_title_tags())) {
            Webtexttool::set_title_tag($titleTag);
        }

        $socialSwitch = $jinput->json->get('socialSwitch');
        if (isset ($socialSwitch) && in_array($socialSwitch, array(true, false))) {
            if ($socialSwitch) {
                Webtexttool::wtt_social_plugin_enable();
            } else {
                Webtexttool::wtt_social_plugin_disable();
            }
        }
    }

    public function onAfterRender() {
        $document = JFactory::getDocument();

        if (JFactory::getApplication()->isAdmin() && version_compare(JVERSION, '3.2', 'ge')) {
            // are we on an edit page?
            $option = JRequest::getCmd('option');
            $view   = JRequest::getCmd('view');
            $layout = JRequest::getCmd('layout');
            $id     = JRequest::getCmd('id');
            if (/* ($option == 'com_k2' && ($view == 'item' || $view == 'category')) || */
                ($layout == 'edit' && (($option == 'com_content'    && $view == 'article') ||
                                       ($option == 'com_categories' && $view == 'category')))) {
                // variations in field names
                if ($option == 'com_k2') {
                    $id     = JRequest::getCmd('cid');
                    if ($view == 'category') {
                        $titleFieldName    = 'name';
                        $metaDescFieldName = 'params[catMetaDesc]';
                        $metaKeyFieldName  = 'params[catMetaKey]';
                    } else {
                        $titleFieldName    = 'title';
                        $metaDescFieldName = 'metadesc';
                        $metaKeyFieldName  = 'metakey';
                        }
                } else {
                    $titleFieldName        = 'jform_title';
                    $metaDescFieldName     = 'jform_metadesc';
                    $metaKeyFieldName      = 'jform_metakey';
                }
                // filter '[' and ']' (used in K2 categories)
                $metaDescFieldNameOrig = $metaDescFieldName;
                $metaKeyFieldNameOrig  = $metaKeyFieldName;
                $metaDescFieldName     = str_replace(array('[', ']'), '', $metaDescFieldName);
                $metaKeyFieldName      = str_replace(array('[', ']'), '', $metaKeyFieldName);

                $currentLanguage         = JFactory::getLanguage();
                $currentLanguageTagArray = explode('-', $currentLanguage->getTag());
                $currentLanguageTag      = $currentLanguageTagArray[0];

                // FIXME: do this for K2 too
                $articleOrCategory = ($view == 'category' ? JTable::getInstance("category") : JTable::getInstance("content") );
                $articleOrCategory->load($id);
                $articleLanguageTag = $articleOrCategory->get("language");
                if ($articleLanguageTag != '*') {
                    $articleLanguageTagArray = explode ('-', $articleLanguageTag);
                    $currentLanguageTag = $articleLanguageTagArray[0];
                }

                // attach our stuff
                if ($document->getType() == 'html') {
                    $page = JResponse::getBody();
                    // insert css and js
                    $ourCssAndJs = '<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.6.3/css/font-awesome.min.css" type="text/css" />' . "\n" .
                        '<link rel="stylesheet" href="' . Webtexttool::get_plugin_path() . 'css/wtt-global-page.min.css" type="text/css" />' . "\n" .
                        '<link rel="stylesheet" href="' . Webtexttool::get_plugin_path() . 'css/wtt-core.min.css" type="text/css" />' . "\n" .
                        '<link rel="stylesheet" href="' . Webtexttool::get_plugin_path() . 'css/wtt-core-2.min.css" type="text/css" />' . "\n" .
                        '<link rel="stylesheet" href="' . Webtexttool::get_plugin_path() . 'css/webtexttool.min.css" type="text/css" />';
                    ob_start();
                    // put all our javascript in an anonymous function call to avoid conflicts
                    // see http://www.mattburkedev.com/multiple-angular-versions-on-the-same-page/
                    ?>

                    <script type="text/javascript">
                    (function() {
                        var existingWindowDotAngular = window.angular,
                            angular = (window.angular = {}),
                            wtt_globals = {
                                            'wtt_short_url':     '<?php echo Webtexttool::wtt_short_url; ?>',
                                            'wtt_base_api_url':  '<?php echo Webtexttool::wtt_api_base_url; ?>/',
                                            'pluginsUrl':        '<?php echo Webtexttool::get_plugin_path(); ?>',
                                            'view':              '<?php echo $view; ?>',
                                            'permalink':         '<?php echo Webtexttool::get_page_url(); ?>',
                                            'siteUrl':           '<?php echo Webtexttool::get_site_url(); ?>',
                                            'titleTag':          '<?php echo Webtexttool::get_title_tag(); ?>',
                                            'titleFieldName':    '<?php echo $titleFieldName; ?>',
                                            'metaDescFieldName': '<?php echo $metaDescFieldName; ?>',
                                            'currentLanguage':   '<?php echo $currentLanguageTag ;?>',
                                            'metaKeyFieldName':  '<?php echo $metaKeyFieldName; ?>',
                                            'metaKeyFieldShown': '<?php echo 'true'; ?>'
                                          };

                        jQuery(function() {
                            wtt_globals.wtt_page_score                   = document.getElementsByClassName('wtt_page_score')[0];
                            wtt_globals.wtt_post_languagecode            = document.getElementsByClassName('wtt_post_languagecode')[0];
                            wtt_globals.wtt_post_synonyms                = document.getElementsByClassName('wtt_post_synonyms')[0];
                            wtt_globals.wtt_content_quality_settings     = document.getElementsByClassName('wtt_content_quality_settings')[0];
                            wtt_globals.wtt_content_quality_suggestions  = document.getElementsByClassName('wtt_content_quality_suggestions')[0];
                            wtt_globals.getCQSettings                    = (wtt_globals.wtt_content_quality_settings.value ? JSON.parse(wtt_globals.wtt_content_quality_settings.value) : '');
                            wtt_globals.getLastSuggestions               = (wtt_globals.wtt_content_quality_suggestions.value ? JSON.parse(wtt_globals.wtt_content_quality_suggestions.value) : '');
                            wtt_globals.pageSCore                        = (wtt_globals.wtt_page_score.value ? JSON.parse(wtt_globals.wtt_page_score.value) : '');
                            if (document.getElementById(wtt_globals.metaKeyFieldName)) {
                                document.getElementById('wtt-keyword').value  = document.getElementById(wtt_globals.metaKeyFieldName).value;
                            } else {
                                wtt_globals.metaKeyFieldShown = false;
                            }
                            jQuery('#wtt-keyword').on('change', function(event){
                                document.getElementById(wtt_globals.metaKeyFieldName).value = this.value;
                            });
                            if (wtt_globals.wtt_post_languagecode.value != '') {
                                document.getElementById('wtt-language-code-field').value = wtt_globals.wtt_post_languagecode.value;
                            } else {
                                document.getElementById('wtt-language-code-field').value = wtt_globals.currentLanguage;
                            }
                        });

                        <?php
                        include dirname(__FILE__) . '/js/underscore-min.js';
                        echo "\n";
                        include dirname(__FILE__) . '/js/wtt-core.min.js';
                        echo "\n";
                        include dirname(__FILE__) . '/js/observeDom.js';
                        echo "\n";
                        include dirname(__FILE__) . '/js/getHtmlContent.js';
                        echo "\n";
                        include dirname(__FILE__) . '/inc/webtexttool-coresetup.min.js';
                        ?>

                        angular.element(document).ready(function() {
                            <?php if ($option == 'com_k2') { ?>
                                // switch these blocks
                                var k2Editor        = document.getElementById('k2ItemFormEditor'),
                                    wttEditor       = document.getElementById('postwebtexttool'),
                                    editorContainer = k2Editor.parentNode;
                                editorContainer.insertBefore(k2Editor, wttEditor);
                                // some more to get it right
                                jQuery("[name='<?php echo $metaDescFieldNameOrig; ?>']").attr('id', '<?php echo $metaDescFieldName; ?>');
                                jQuery("[name='<?php echo $metaKeyFieldNameOrig ; ?>']").attr('id', '<?php echo $metaKeyFieldName; ?>');
                            <?php } ?>
                            angular.bootstrap(document.getElementById('wtt-dashboard'), ['wttDashboard']);
                        window.angular = existingWindowDotAngular;
                        });
                    })();
                    </script>

                    <?php
                    $ourCssAndJs .= ob_get_contents();
                    ob_end_clean();
                    $page = str_replace('</head>', $ourCssAndJs . '</head>', $page);

                    // insert custom html
                    ob_start();
                    include dirname(__FILE__) . '/inc/webtexttool-core.php';
                    include dirname(__FILE__) . '/inc/webtexttool-language.php';
                    $ourHtml = ob_get_contents();
                    ob_end_clean();

                    if ($option == 'com_k2') {
                        // for K2
                        $page = str_replace('<div class="k2ItemFormEditor">', $ourHtml . '<div class="k2ItemFormEditor row-fluid span9" id="k2ItemFormEditor">', $page, $count);
                    } else {
                        // for Isis
                        $page = str_replace('<fieldset class="form-vertical">', '<fieldset class="form-vertical">' . $ourHtml, $page, $count);
                        // for Hathor
                        if ($count === 0) {
                            // only first occurrence
                            $needle = 'class="pane-sliders">';
                            $pos = strpos($page, $needle);
                            if ($pos !== false) {
                                $page = substr_replace($page, $needle . $ourHtml, $pos, strlen($needle));
                            }
                        }
                    }

                    JResponse::setBody($page);
                }
            }
        }
    }

}
