<?php
/**
 * Webtexttool - SEO made easy, for everyone
 *
 * @author       Jan Martin Roelofs
 * @copyright    (c) Webtexttool - 2019
 * @package      webtexttool
 * @license      http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version      2.0.1
 * @date         2019-04-06
 *
 */

defined('_JEXEC') or die;

/**
 * webtexttool social system plugin
 */

class plgSystemWebtexttoolSocial extends JPlugin {

    private function renderTag($name, $value, $type = 'name') {
        if ($value != '') {
            $document = JFactory::getDocument();
            if ($type == 'name') {
                $document->setMetadata(htmlspecialchars($name), htmlspecialchars($value));
            } else if ($type == 'property') {
                $document->addCustomTag('<meta property="'.htmlspecialchars($name).'" content="' . htmlspecialchars($value) . '" />');
            } else if ($type == 'itemprop') {
                $document->addCustomTag('<meta itemprop="'.htmlspecialchars($name).'" content="' . htmlspecialchars($value) . '" />');
            }
        }
    }

    private function getPlgId(){
        // stupid hack since there doesn't seem to be another way to get plugin id
        $db = JFactory::getDBO();
        $sql = 'SELECT `extension_id` FROM `#__extensions` WHERE `element` = "webtexttoolsocial" AND `folder` = "system"'; // check the #__extensions table if you don't know your element / folder
        $db->setQuery($sql);
        if( !($plg = $db->loadObject()) ){
            return false;
        } else {
            return (int) $plg->extension_id;
        }
    }

    private function writeParam($param, $value) {
        $db = JFactory::getDbo();
        $extension = new JTableExtension($db);
        $ext_id = $this->getPlgId();
        // get the existing extension data
        $extension->load($ext_id);
        $this->params->set($param, $value);
        $extension->bind( array('params' => $this->params->toString()) );
        // check and store
        if (!$extension->check()) {
            $this->setError($extension->getError());
            return false;
        }
        if (!$extension->store()) {
            $this->setError($extension->getError());
            return false;
        }
        return ($value);
    }

   	function onAjaxWebtexttoolSocial() {
        JSession::checkToken('get') or die ('Invalid Token');
        $jinput = JFactory::getApplication()->input;

        $pluginsSwitchParams = array(
            'facebook',
            'twitter',
            'google'
        );
        foreach ($pluginsSwitchParams as $switch) {
            $input = $jinput->json->get($switch);
            if (isset ($input) && in_array($input, array(true, false))) {
                return $this->writeParam($switch, (int)$input);
            }
        }

        $pluginsTextParams = array(
            'facebookPage',
            'facebookAdminId',
            'facebookPlatformAppId',
            'twitterUsername',
            'googlePlusPage'
        );
        foreach ($pluginsTextParams as $switch) {
            $input = $jinput->json->get($switch);
            if (isset ($input)) {
                return $this->writeParam($switch, $input);
            }
        }
    }

    function onBeforeRender() {
        $app    = JFactory::getApplication();
        $option	= $app->input->get('option');
        $format = $app->input->get('format');
        $view   = $app->input->get('view');

        if (version_compare(JVERSION, '3.2', 'lt') ||
                !JPluginHelper::isEnabled('system', 'webtexttool') ||
                $format == 'feed' ||
                $format == 'pdf' ||
                $format == 'json' ||
                $app->getName() != 'site' ||
                ($option != 'com_content' && $option != 'com_k2')) {
            return;
        }

        $facebook = $this->params->get('facebook');
        $twitter  = $this->params->get('twitter');
        $google   = $this->params->get('google');

        $document = JFactory::getDocument();
        $config   = JFactory::getConfig();

        if ($facebook) {
            // Site Name
            $this->renderTag('og:site_name', $config->get('sitename'), 'property');
            $this->renderTag('og:title', $document->title, 'property');
            $this->renderTag('og:description', $document->description, 'property');
            $this->renderTag('og:url', $document->base, 'property');
            $this->renderTag('og:type', ($view == 'item' || $view == 'article' ? 'article' : 'website'), 'property');
            $this->renderTag('article: publisher', $this->params->get('facebookPage'), 'property');
            $this->renderTag('fb:admins', $this->params->get('facebookAdminId'), 'property');
            $this->renderTag('fb:app_id', $this->params->get('facebookPlatformAppId'), 'property');
        }

        if ($twitter) {
            // Site Name
            $this->renderTag('twitter:title', $document->title, 'name');
            $this->renderTag('twitter:description', $document->description, 'name');
            $this->renderTag('twitter:url', $document->base, 'name');
            if ($this->params->get('twitterUsername') != ''){
                $this->renderTag('twitter:site', '@' . $this->params->get('twitterUsername'), 'name');
            }
        }

        if ($google) {
            // Site Name
            $this->renderTag('name', $document->title, 'itemprop');
            if ($this->params->get('googlePlusPage') != ''){
                $document = JFactory::getDocument();
                $document->addCustomTag('<link rel="publisher" href="' . $this->params->get('googlePlusPage') . '" />');
            }
        }
    }
}
